#  Copyright (c) 2020. Sinri Edogawa

MYSQL_CONDITION_OP_EQ = "="
MYSQL_CONDITION_OP_GT = ">"
MYSQL_CONDITION_OP_EGT = ">="
MYSQL_CONDITION_OP_LT = "<"
MYSQL_CONDITION_OP_ELT = "<="
MYSQL_CONDITION_OP_NEQ = "<>"
MYSQL_CONDITION_OP_NULL_SAFE_EQUAL = "<=>"
MYSQL_CONDITION_OP_IS = "IS"
MYSQL_CONDITION_OP_IS_NOT = "IS NOT"
MYSQL_CONDITION_OP_IN = "IN"
MYSQL_CONDITION_OP_NOT_IN = "NOT IN"
MYSQL_CONDITION_OP_LIKE = "LIKE"
MYSQL_CONDITION_OP_NOT_LIKE = "NOT LIKE"
MYSQL_CONDITION_OP_BETWEEN = "BETWEEN"
MYSQL_CONDITION_OP_NOT_BETWEEN = "NOT BETWEEN"

MYSQL_CONDITION_MACRO_IS_NULL_OR_EMPTY_STRING = "MACRO_IS_NULL_OR_EMPTY_STRING"
MYSQL_CONDITION_MACRO_IS_NOT_NULL_NOR_EMPTY_STRING = "MACRO_IS_NOT_NULL_NOR_EMPTY_STRING"

MYSQL_CONDITION_MACRO_RAW_EXPRESSION = "MACRO_RAW_EXPRESSION"

MYSQL_CONDITION_OP_EXISTS = "EXISTS"
MYSQL_CONDITION_OP_NOT_EXISTS = "NOT EXISTS"

MYSQL_CONDITION_OP_PARENTHESES_AND = "AND"
MYSQL_CONDITION_OP_PARENTHESES_OR = "OR"

MYSQL_CONDITION_CONST_TRUE = "TRUE"
MYSQL_CONDITION_CONST_FALSE = "FALSE"
MYSQL_CONDITION_CONST_NULL = "NULL"

MYSQL_CONDITION_LIKE_LEFT_WILDCARD = "LIKE_LEFT_WILDCARD"
MYSQL_CONDITION_LIKE_RIGHT_WILDCARD = "LIKE_RIGHT_WILDCARD"
MYSQL_CONDITION_LIKE_BOTH_WILDCARD = "LIKE_BOTH_WILDCARD"
